function [x_opt]=rounding2(x0,w,M,n,options)
%% rounding algorithm is Algorithm 4.2 in the paper
% AN EFFICIENT QUADRATIC PROGRAMMING RELAXATION BASED ALGORITHM FOR 
% LARGE-SCALE MIMO DETECTION 
% by Ping-Fan Zhao, Qing-Na Li, Wei-Kun Chen, and Ya-Feng Liu
% to appear in SIAM Journal on Optimization, 2021.

% Send your comments and suggestions to    %%%%%%
%             qnl@bit.edu.cn               %%%%%%

%%%%%%%%% This version: February, 2021     %%%%%%%


%% input:
% x0 is the solution obtained by PNQP.m

% w is the linear term coefficient in the objective function of the RSQP model;

% M means M-Phase-Shift Keying (M-PSK) modulation;

% n means the length of transmitted signals;

% options: parameters and other information:

% options.a and options.b: M-dimensional vectors defined as: 
% p = 0:M-1; a = (cos(2*pi/M.*p))'; b = (sin(2*pi/M.*p))'; 

% options.Qr and options.Qi: Qe = H'*H; Qr = real(Qe) - diag(diag(real(Qe))); Qi = imag(Qe);
% where H is an m x n-dimensional channel matrix

%% output:
% x_opt is an M x n-dimensional matrix. reshape(T_rsqp2,n*M,1) is the final 
% solution of the RSQP problem after rounding.



a = options.a;
b = options.b;
Qr = options.Qr;
Qi = options.Qi;

X = reshape(x0,M,n)';
Matval = reshape(w,M,n);
Xa = X*a;
Xb = X*b;

for j = 1:n
    
    qrj = Qr(j,:);
    qij = Qi(j,:);
    Qjra = qrj*Xa;
    Qjib =qij*Xb;
    Qjia =qij *Xa;
    Qjrb = qrj*Xb;
    
    
    gj = a* (Qjra-Qjib);
    gj = gj + b*(Qjia+Qjrb);
    gj = gj + Matval(:,j);
    gj = gj*2;
    
    [~,s] = min(gj);
    
    X(j,:) = 0; X(j,s) = 1;
    
    
    Xa(j) = a(s); Xb(j) = b(s);
    
end
x_opt = X';
