% This file is to demonstrate how to use PN-QP method to solve MIMO
% detection problem.

% Basic parameters
%rand('seed',0);

n = 512; % the length of transmitted signals
m = n; % the length of received signals
M = 8; % M-Phase-Shift Keying (M-PSK) modulation
snr = 12;
sigma = sqrt(n*10.^(-(snr) / 10 ));
nM = n*M;


% Loop multiple times
iter = 1;
disp = 1;
maxit = 1;

t1 = zeros(maxit,1);
ser_rsqp = zeros(maxit,1);

while iter<=maxit
    % Randomly generate channel and noise
    rH = randn(m,n);
    iH = randn(m,n);
    H = rH+1i*iH; %H
    clear rH iH
    rv = normrnd(0,sigma,m,1);
    iv = normrnd(0,sigma,m,1);
    v = rv + 1i*iv; %v
    clear rv iv
    
    % Randomly generate instances
    s = randi([0 M-1],n,1);
    x = exp(1i*2/M*pi.*s); %x
    r = H*x + v; %r
    
    [G,w,options] = getparameters(M,n,H,r);
    
    % Solve
    
    if disp
         fprintf('******************----- PN-QP for MIMO starts-----********************\n');
         fprintf(' Below is the basic information:\nThe length of signals n = %d, number of observations m = %d\n', n,m);
         fprintf('M-Phase-Shift Keying (M-PSK) modulation M = %d, Signal Noise Ratio snr = %d\n',M,snr);
        % fprintf('...........\n',M,snr);
    end
    tic;
    [xx0,info] = PNQP(G,w,M,n,options);
    [T_rsqp2] = rounding(xx0,w,M,n,options); % obtain the result by our rounding procedure for MIMO, i.e., the matrix form of the final solution
    t1(iter) = toc;
    [~,k] = max(T_rsqp2);
    s1 = k-1;
    %     t=reshape(T_rsqp2,nM,1); % the final solution
    
    % SER
    num_rsqp = 0;
    for iterea = 1:n
        if s1(iterea) ~= s(iterea)
            num_rsqp = num_rsqp+1;
        end
    end
    ser_rsqp(iter) = num_rsqp/n;
    
    if disp
        fprintf('\nThe algorithm stops! The final information is as follows:\n');
        fprintf('The number of iterations: %d; The number of total inner iterations in cg: %d\n', info.iter,info.iter_cg);
        fprintf('The cputime is %5.3f,   the symbol error rate SER is %5.3f\n', t1(iter), ser_rsqp(iter))
        fprintf('*******************************************************************\n\n\n');
    end

    iter = iter + 1;
end
%fprintf('m = %d, n = %d, M = %d, repeat times %d, SER = %9.8f\n', m, n , M, maxit, ser_rsqp(iter))
function [G,w,options] = getparameters(M,n,H,r)
p = 0:M-1;
a = cos(2*pi/M.*p);
b = sin(2*pi/M.*p);
In = eye(n);
A = kron(In,a);
B = kron(In,b);
P = [A;B]; %P
P = sparse(P);
EM = ones(M,M);
Mek = kron(In,EM);
Mek = sparse(Mek);
clear EM

Qe = H'*H; %Q
Q = [real(Qe),-imag(Qe);imag(Qe),real(Qe)]; %Q^
Qr = real(Qe) - diag(diag(real(Qe)));
Qi = imag(Qe);

ce = -H'*r; %c
c = [real(ce);imag(ce)]; %c^

Ge = P'*Q*P; %G
w = (c'*P)'; %w
G = Ge;
for j = 1:n
    G((1 + (j - 1)*M):j*M,(1 + (j - 1)*M):j*M) = 0;
end %\widetilde{G} in the paper

options.a = a';
options.b = b';
options.Mek = Mek;
options.Qr = Qr;
options.Qi = Qi;
end